@@ -1,13 +1,17 @@
 import abc
 import struct
 from abc import ABCMeta
+from asyncio import Future
 from enum import IntEnum, unique
-from typing import Tuple, Optional
+from typing import Tuple, Optional, Union
 
 from rsocket.error_codes import ErrorCode
 from rsocket.exceptions import RSocketProtocolError, ParseError, RSocketUnknownFrameType
+from rsocket.fragment import Fragment
+from rsocket.frame_fragmenter import data_to_fragments_if_required
 from rsocket.frame_helpers import is_flag_set, unpack_position, pack_position, unpack_24bit, pack_24bit, unpack_32bit, \
     ensure_bytes, pack_string, unpack_string
+from rsocket.logger import logger
 
 PROTOCOL_MAJOR_VERSION = 1
 PROTOCOL_MINOR_VERSION = 0
@@ -25,6 +29,8 @@ _FLAG_RESUME_BIT = 0x80
 _FLAG_RESPOND_BIT = 0x80
 _FLAG_NEXT_BIT = 0x20
 
+MINIMUM_FRAGMENT_SIZE_BYTES = 64
+
 
 @unique
 class FrameType(IntEnum):
@@ -58,35 +64,36 @@ class Header:
     )
 
 
+def is_blank(value: Optional[bytes]) -> bool:
+    return value is None or len(value) == 0
+
+
 def parse_header(frame: Header, buffer: bytes, offset: int) -> int:
     frame.length = len(buffer)
     frame.stream_id, frame.frame_type, flags = struct.unpack_from('>IBB', buffer, offset)
     flags |= (frame.frame_type & 3) << 8
     frame_type_id = frame.frame_type >> 2
-    frame.frame_type = FrameType(frame_type_id)
+
+    try:
+        frame.frame_type = FrameType(frame_type_id)
+    except ValueError as exception:
+        raise RSocketUnknownFrameType(frame_type_id) from exception
+
     frame.flags_ignore = is_flag_set(flags, _FLAG_IGNORE_BIT)
     frame.flags_metadata = is_flag_set(flags, _FLAG_METADATA_BIT)
     return flags
 
 
-class FragmentableFrame:
-    __slots__ = (
-        'metadata',
-        'data',
-        'flags_follows',
-        'flags_complete',
-        'flags_next',
-        'stream_id'
-    )
-
-
 class Frame(Header, metaclass=ABCMeta):
     __slots__ = (
         'metadata',
         'data',
         'flags_follows',
         'flags_complete',
-        'metadata_only'
+        'metadata_only',
+        'sent_future',
+        'fragment_size_bytes',
+        'fragment_generator'
     )
 
     def __init__(self, frame_type: FrameType):
@@ -100,9 +107,12 @@ class Frame(Header, metaclass=ABCMeta):
         self.flags_metadata = False
         self.flags_follows = False
         self.flags_complete = False
-
         self.metadata_only = False
 
+        self.fragment_size_bytes: Optional[int] = None
+        self.fragment_generator = None
+        self.sent_future: Optional[Future] = None
+
     def parse_metadata(self, buffer: bytes, offset: int) -> int:
         if not self.flags_metadata:
             return 0
@@ -178,6 +188,30 @@ class Frame(Header, metaclass=ABCMeta):
 
         return length
 
+    def __str__(self):
+        return str(f'({FrameType(self.frame_type).name},{self.data},{self.metadata},{self.flags_complete})')
+
+
+class FrameFragmentMixin(metaclass=abc.ABCMeta):
+
+    def get_next_fragment(self, requires_length_header: bool = True) -> Optional['Frame']:
+        if self.fragment_generator is None:
+            self.fragment_generator = data_to_fragments_if_required(
+                self.data,
+                self.metadata,
+                get_header_length(self),
+                self.fragment_size_bytes,
+                requires_length_header
+            )
+
+        try:
+            fragment = self.fragment_generator.__next__()
+            return new_frame_fragment(self, fragment)
+        except GeneratorExit:
+            return None
+        except StopIteration:
+            return None
+
 
 class SetupFrame(Frame):
     __slots__ = (
@@ -351,20 +385,19 @@ class RequestFrame(Frame):
         offset += self.parse_data(buffer, offset)
 
 
-class RequestResponseFrame(RequestFrame):
+class RequestResponseFrame(RequestFrame, FrameFragmentMixin):
     __slots__ = ()
 
     def __init__(self):
         super().__init__(FrameType.REQUEST_RESPONSE)
 
-    # noinspection PyAttributeOutsideInit
     def parse(self, buffer, offset):
         header = RequestFrame.parse(self, buffer, offset)
         offset += header[0]
         self._parse_payload(buffer, offset)
 
 
-class RequestFireAndForgetFrame(RequestFrame):
+class RequestFireAndForgetFrame(RequestFrame, FrameFragmentMixin):
     __slots__ = ()
 
     def __init__(self):
@@ -376,7 +409,7 @@ class RequestFireAndForgetFrame(RequestFrame):
         self._parse_payload(buffer, offset)
 
 
-class RequestStreamFrame(RequestFrame):
+class RequestStreamFrame(RequestFrame, FrameFragmentMixin):
     __slots__ = 'initial_request_n'
 
     def __init__(self):
@@ -395,7 +428,7 @@ class RequestStreamFrame(RequestFrame):
         return RequestFrame.serialize(self, middle)
 
 
-class RequestChannelFrame(RequestFrame):
+class RequestChannelFrame(RequestFrame, FrameFragmentMixin):
     __slots__ = (
         'initial_request_n',
         'flags_complete',
@@ -453,7 +486,7 @@ class CancelFrame(Frame):
         parse_header(self, buffer, offset)
 
 
-class PayloadFrame(Frame):
+class PayloadFrame(Frame, FrameFragmentMixin):
     __slots__ = (
         'flags_follows',
         'flags_complete',
@@ -478,12 +511,19 @@ class PayloadFrame(Frame):
     def serialize(self, middle=b'', flags=0):
         flags &= ~(_FLAG_FOLLOWS_BIT | _FLAG_COMPLETE_BIT |
                    _FLAG_NEXT_BIT)
+
         if self.flags_follows:
             flags |= _FLAG_FOLLOWS_BIT
+
         if self.flags_complete:
             flags |= _FLAG_COMPLETE_BIT
+
+        if not is_blank(self.data) or not is_blank(self.metadata):
+            self.flags_next = True
+
         if self.flags_next:
             flags |= _FLAG_NEXT_BIT
+
         return Frame.serialize(self, flags=flags)
 
 
@@ -609,19 +649,27 @@ def parse_or_ignore(buffer: bytes) -> Optional[Frame]:
     header = Header()
     parse_header(header, buffer, 0)
 
-    try:
-        frame = _frame_class_by_id[header.frame_type]()
-    except KeyError as exception:
-        raise RSocketUnknownFrameType(header.frame_type) from exception
+    frame = _frame_class_by_id[header.frame_type]()
 
     try:
         frame.parse(buffer, 0)
-        return frame
+
+        if not is_frame_to_ignore(frame):
+            return frame
+
     except Exception as exception:
         if not header.flags_ignore:
             raise RSocketProtocolError(ErrorCode.CONNECTION_ERROR, str(exception)) from exception
 
 
+def is_frame_to_ignore(frame: Frame) -> bool:
+    if isinstance(frame, MetadataPushFrame) and frame.stream_id != CONNECTION_STREAM_ID:
+        logger().error('Invalid metadata frame')
+        return True
+
+    return False
+
+
 def is_fragmentable_frame(frame: Frame) -> bool:
     return isinstance(frame, (
         PayloadFrame,
@@ -632,6 +680,43 @@ def is_fragmentable_frame(frame: Frame) -> bool:
     ))
 
 
+FragmentableFrame = Union[PayloadFrame,
+                          RequestResponseFrame,
+                          RequestChannelFrame,
+                          RequestStreamFrame,
+                          RequestFireAndForgetFrame]
+
+
+def new_frame_fragment(base_frame: FragmentableFrame, fragment: Fragment) -> Frame:
+    if fragment.is_first:
+        frame = base_frame.__class__()
+    else:
+        frame = PayloadFrame()
+
+    if isinstance(frame, PayloadFrame):
+        if not is_blank(frame.data) or not is_blank(frame.metadata):
+            frame.flags_next = True
+
+    frame.stream_id = base_frame.stream_id
+
+    frame.flags_ignore = base_frame.flags_ignore
+    frame.flags_metadata = base_frame.flags_metadata
+    frame.metadata_only = base_frame.metadata_only
+
+    if hasattr(base_frame, 'initial_request_n'):
+        frame.initial_request_n = base_frame.initial_request_n
+
+    frame.data = fragment.data
+    frame.metadata = fragment.metadata
+    frame.flags_follows = fragment.is_last is False
+
+    if fragment.is_last is None or fragment.is_last:
+        frame.sent_future = base_frame.sent_future
+        frame.flags_complete = base_frame.flags_complete
+
+    return frame
+
+
 def exception_to_error_frame(stream_id: int, exception: Exception) -> ErrorFrame:
     frame = ErrorFrame()
     frame.stream_id = stream_id
@@ -665,3 +750,15 @@ initiate_request_frame_types = (RequestResponseFrame,
                                 RequestChannelFrame,
                                 RequestFireAndForgetFrame
                                 )
+
+frame_header_length = {
+    PayloadFrame: 6,
+    RequestResponseFrame: 6,
+    RequestFireAndForgetFrame: 6,
+    RequestStreamFrame: 10,
+    RequestChannelFrame: 10,
+}
+
+
+def get_header_length(frame: FragmentableFrame) -> int:
+    return frame_header_length[frame.__class__]
