@@ -1,3 +1,4 @@
+import asyncio
 from typing import Optional
 
 from reactivestreams.publisher import Publisher
@@ -10,8 +11,12 @@ from rsocket.rsocket import RSocket
 
 class RequestChannelRequester(RequestChannelCommon):
 
-    def __init__(self, socket: RSocket, payload: Payload, remote_publisher: Optional[Publisher] = None):
-        super().__init__(socket, remote_publisher)
+    def __init__(self,
+                 socket: RSocket,
+                 payload: Payload,
+                 remote_publisher: Optional[Publisher] = None,
+                 sending_done_event: Optional[asyncio.Event] = None):
+        super().__init__(socket, remote_publisher, sending_done_event)
         self._payload = payload
 
     def setup(self):
@@ -19,10 +24,11 @@ class RequestChannelRequester(RequestChannelCommon):
 
     def _send_channel_request(self, payload: Payload):
         self.socket.send_request(
-            to_request_channel_frame(self.stream_id,
-                                     payload,
-                                     self._initial_request_n,
-                                     self._remote_publisher is None)
+            to_request_channel_frame(stream_id=self.stream_id,
+                                     payload=payload,
+                                     initial_request_n=self._initial_request_n,
+                                     complete=self._remote_publisher is None,
+                                     fragment_size_bytes=self.socket.get_fragment_size_bytes())
         )
 
     def subscribe(self, subscriber: Subscriber):
