@@ -1,7 +1,7 @@
 import asyncio
 from asyncio import Task
 from contextlib import contextmanager
-from typing import Any
+from typing import Any, Awaitable
 from typing import TypeVar
 from typing import Union, Callable, Optional, Tuple
 
@@ -9,8 +9,10 @@ from reactivestreams.publisher import DefaultPublisher
 from reactivestreams.subscriber import Subscriber
 from reactivestreams.subscription import DefaultSubscription
 from rsocket.exceptions import RSocketTransportError
+from rsocket.extensions.mimetype import WellKnownType
 from rsocket.frame import Frame
 from rsocket.frame_helpers import serialize_128max_value, parse_type
+from rsocket.local_typing import ByteTypes
 from rsocket.logger import logger
 from rsocket.payload import Payload
 
@@ -27,6 +29,10 @@ def create_future(value: Optional[Any] = _default) -> asyncio.Future:
     return future
 
 
+def create_response(data: Optional[ByteTypes] = None, metadata: Optional[ByteTypes] = None) -> Awaitable[Payload]:
+    return create_future(Payload(data, metadata))
+
+
 def create_error_future(exception: Exception) -> asyncio.Future:
     future = create_future()
     future.set_exception(exception)
@@ -43,21 +49,12 @@ class DefaultPublisherSubscription(DefaultPublisher, DefaultSubscription):
         subscriber.on_subscribe(self)
 
 
-class WellKnownType:
-    __slots__ = (
-        'name',
-        'id'
-    )
+def map_types_by_name(types):
+    return {value.value.name: value.value for value in types}
 
-    def __init__(self, name: bytes, id_: int):
-        self.name = name
-        self.id = id_
 
-    def __eq__(self, other):
-        return self.name == other.name and self.id == other.id
-
-    def __hash__(self):
-        return hash((self.id, self.name))
+def map_types_by_id(types):
+    return {value.value.id: value.value for value in types}
 
 
 @contextmanager
@@ -120,5 +117,11 @@ async def cancel_if_task_exists(task: Optional[Task]):
             await task
         except asyncio.CancelledError:
             logger().debug('Asyncio task cancellation error: %s', task)
-        except RuntimeError:
+        except Exception:
             logger().warning('Runtime error canceling task: %s', task, exc_info=True)
+
+
+def utf8_decode(data: bytes):
+    if data is not None:
+        return data.decode('utf-8')
+    return None
