@@ -1,5 +1,5 @@
 import asyncio
-from asyncio import Future, CancelledError
+from asyncio import CancelledError
 from datetime import timedelta, datetime
 from typing import Optional, Callable, AsyncGenerator, Any
 from typing import Union
@@ -8,6 +8,7 @@ from reactivestreams.publisher import Publisher
 from rsocket.exceptions import RSocketNoAvailableTransport
 from rsocket.extensions.mimetypes import WellKnownMimeTypes
 from rsocket.helpers import create_future, cancel_if_task_exists
+from rsocket.local_typing import Awaitable
 from rsocket.logger import logger
 from rsocket.payload import Payload
 from rsocket.request_handler import BaseRequestHandler
@@ -24,11 +25,12 @@ class RSocketClient(RSocketBase):
                  honor_lease=False,
                  lease_publisher: Optional[Publisher] = None,
                  request_queue_size: int = 0,
-                 data_encoding: Union[bytes, WellKnownMimeTypes] = WellKnownMimeTypes.APPLICATION_JSON,
-                 metadata_encoding: Union[bytes, WellKnownMimeTypes] = WellKnownMimeTypes.APPLICATION_JSON,
+                 data_encoding: Union[str, bytes, WellKnownMimeTypes] = WellKnownMimeTypes.APPLICATION_JSON,
+                 metadata_encoding: Union[str, bytes, WellKnownMimeTypes] = WellKnownMimeTypes.APPLICATION_JSON,
                  keep_alive_period: timedelta = timedelta(milliseconds=500),
                  max_lifetime_period: timedelta = timedelta(minutes=10),
                  setup_payload: Optional[Payload] = None,
+                 fragment_size_bytes: Optional[int] = None
                  ):
         self._transport_provider = transport_provider.__aiter__()
         self._is_server_alive = True
@@ -47,9 +49,10 @@ class RSocketClient(RSocketBase):
                          metadata_encoding=metadata_encoding,
                          keep_alive_period=keep_alive_period,
                          max_lifetime_period=max_lifetime_period,
-                         setup_payload=setup_payload)
+                         setup_payload=setup_payload,
+                         fragment_size_bytes=fragment_size_bytes)
 
-    def _current_transport(self) -> Future:
+    def _current_transport(self) -> Awaitable[Transport]:
         return self._next_transport
 
     def _log_identifier(self) -> str:
@@ -63,9 +66,12 @@ class RSocketClient(RSocketBase):
 
         try:
             await self._connect_new_transport()
+        except RSocketNoAvailableTransport:
+            logger().error('%s: No available transport', self._log_identifier(), exc_info=True)
+            return
         except Exception as exception:
             logger().error('%s: Connection error', self._log_identifier(), exc_info=True)
-            await self._on_connection_lost(exception)
+            await self._on_connection_error(exception)
             return
 
         return await super().connect()
@@ -93,6 +99,7 @@ class RSocketClient(RSocketBase):
         await self._close()
 
     async def _close(self, reconnect=False):
+
         if not reconnect:
             await cancel_if_task_exists(self._reconnect_task)
         else:
@@ -108,23 +115,28 @@ class RSocketClient(RSocketBase):
         return 1
 
     async def reconnect(self):
+        logger().info('%s: Reconnecting', self._log_identifier())
+
         self._connect_request_event.set()
 
     async def _reconnect_listener(self):
         try:
             while True:
-                await self._connect_request_event.wait()
+                try:
+                    await self._connect_request_event.wait()
 
-                logger().debug('%s: Got reconnect request', self._log_identifier())
+                    logger().debug('%s: Got reconnect request', self._log_identifier())
 
-                if self._connecting:
-                    continue
+                    if self._connecting:
+                        continue
 
-                self._connecting = True
-                self._connect_request_event.clear()
-                await self._close(reconnect=True)
-                self._next_transport = create_future()
-                await self.connect()
+                    self._connecting = True
+                    self._connect_request_event.clear()
+                    await self._close(reconnect=True)
+                    self._next_transport = create_future()
+                    await self.connect()
+                finally:
+                    self._connect_request_event.clear()
         except CancelledError:
             logger().debug('%s: Asyncio task canceled: reconnect_listener', self._log_identifier())
         except Exception:
