@@ -4,17 +4,16 @@ from typing import Optional, Union
 from rsocket.exceptions import RSocketValueError
 from rsocket.frame import Frame, MAX_REQUEST_N
 from rsocket.frame_builders import to_cancel_frame, to_request_n_frame
-from rsocket.logger import logger
 from rsocket.rsocket import RSocket
 from rsocket.rsocket_internal import RSocketInternal
 from rsocket.streams.backpressureapi import BackpressureApi
 
 
 class StreamHandler(BackpressureApi, metaclass=ABCMeta):
-    def __init__(self, socket):
+    def __init__(self, socket: Union[RSocket, RSocketInternal]):
         super().__init__()
         self.stream_id: Optional[int] = None
-        self.socket: Union[RSocket, RSocketInternal] = socket
+        self.socket = socket
         self._initial_request_n = MAX_REQUEST_N
 
     @abstractmethod
@@ -38,14 +37,11 @@ class StreamHandler(BackpressureApi, metaclass=ABCMeta):
 
     def send_cancel(self):
         """Convenience method for use by requester subclasses."""
-        logger().debug('Sending cancel')
 
         self.socket.send_frame(to_cancel_frame(self.stream_id))
         self._finish_stream()
 
     def send_request_n(self, n: int):
-        logger().debug('Sending request N: %d', n)
-
         self.socket.send_frame(to_request_n_frame(self.stream_id, n))
 
     def _finish_stream(self):
